pro mir_save,fn,filename=filename, nowait=nowait, new=new
;yes
;=Task:MIR_SAVE --- To save the dataset in IDL format.
;#Type: i/o
;+Use:
;      After some repairs and calibrations on the data, the data set 
;      and all the environment variables of MIR can be saved to disk
;      with the MIR_SAVE commnad in IDL format. MIR by default saves
;      data to the current directory, assuming mysaveset is a filename
;      unless the filename begins with a slash (/). In this case MIR
;      assumes that the address is  the full pathname. If no filename
;      is specified, MIR will make up a filename based on the date
;      and time. This is convenient for unstable systems since the
;      files are  named sequentially. The saved data can be restored
;      by the command MIR_RESORE
;@filename:
;      The filename of the data to be saved. For example, to save the
;      file 'mysaveset', you can try
;      >mir_save,'mysaveset'   or
;      >mir_save,'mysaveset'   or
;      >mir_save,filename='mysaveset'
;&history:
;--------------------------------------------------------------------
;      cykuo 25dec03 adapting the header
;--------------------------------------------------------------------

;Define common blocks
common wlm
common data_set
common global
common plo

if (e.campuslogin ne 'sma' and e.campuslogin ne 'cfa') then begin

; this section for caltech/ovro

  filename=e.dataset_dir+'/dataset/'+e.save_fn
  test = CALL_EXTERNAL('db.so', 'db_check_dataset_dir', e.dataset_dir)
  if(test EQ -1) then begin
      print, "Unable to create dataset directory"
      return
  endif

endif  else begin        

;this section for cfa/sma 

;if there was no filename on the command line then make up a name
  if (keyword_set(filename) eq 0 and n_params() eq 0) then begin
    date_now = systime(0)
    parts=strtrim(strsplit(date_now,' ',/extract),2L)
    parts=parts(where (parts ne ''))
    parts(3)=strmid(parts(3),0,strpos(parts(3),':',/reverse_search))
    e.save_fn='saveset'
    for i=0L,n_elements(parts)-2 do begin
       e.save_fn=e.save_fn+'_'+parts(i)
    endfor
  endif

if (n_params() ne 0 ) then e.save_fn = fn
if (keyword_set(filename) eq 1) then e.save_fn = filename
if (strmid(e.save_fn,0,1) eq '/') then begin
  filename=e.save_fn
endif else begin
  filename=e.dataset_dir+e.save_fn
endelse


  print,'All data and the idl environment will be saved to file: '
  print,filename
  newfilename='a'
  if keyword_set(nowait) then newfilename='' else read,newfilename,prompt='Return for OK, or enter another file name: '
  if newfilename ne '' then begin
    length = strlen(newfilename)
    if (strmid(newfilename,0,1) eq "'") then  $
      newfilename = strmid(newfilename,1,length-1)
    if (strmid(newfilename,length-2,1) eq "'") then  $
      newfilename = strmid(newfilename,0,length-2)

    filename = newfilename
    e.save_fn = filename
  endif

endelse

if keyword_set(new) then begin
   temp_pc=make_array(n_elements(psf),/long,value=0L)
   pt_first=0L
   pt_npts=sp[psf[0]].nch
   temp_ch=ch[pcf[0]:pcf[0]+pt_npts-1]
   pt_first=pt_first+pt_npts
   for i=1L, (n_elements(psf)-1L) do begin
      pt_npts=sp[psf[i]].nch
      temp_pc[i]=pt_first
      temp_ch=[temp_ch,ch[pcf[i]:pcf[i]+pt_npts-1]]
      pt_first=pt_first+pt_npts
   endfor

   ch=temp_ch
   pc=temp_pc
   sp=sp[psf]
   re.integs=re.integs[prf]
   re.toffs=re.toffs[prf]
   re.noises=re.noises[prf]
   re.scales=re.scales[prf]
   re.wts=re.wts[prf]

   pi = make_array(n_elements(sp),/long,value=0)
   pb = make_array(n_elements(sp),/long,value=0)
   ps = make_array(n_elements(sp),/long,value=0)
   in_prev=-1L
   bl_prev=-1L
   in_index=0L
   bl_index=0L
   in_max=n_elements(in)-1L
   bl_max=n_elements(bl)-1L
   for i=0L,(n_elements(sp)-1L) do begin
      if (sp(i).inhid ne in_prev) then begin
         in_upper= min([in_index+2L,in_max])
         in_index = in_index + $
           max([where(in(in_index:in_upper).inhid eq sp(i).inhid,count)])
         if (count le 0) then in_index = max([where(in.inhid eq sp(i).inhid)])
;print,'here:',in_index
         in_prev = sp(i).inhid
      endif
      pi(i) = in_index
      if (sp(i).blhid ne bl_prev) then begin
         bl_index= max([0L,bl_index-45L])
         bl_upper= min([bl_index+90L,bl_max])
         bl_index = bl_index + $
           max([where(bl(bl_index:bl_upper).blhid eq sp(i).blhid,count)])
         if (count le 0) then bl_index = max([where(bl.blhid eq sp(i).blhid)])
         bl_prev = sp(i).blhid
      endif
      pb(i) = bl_index
      ps(i) = i
   endfor
   pif=pi
   pbf=pb
   psf=ps
   prf=ps
   pis=pi
   pbs=pb
   pss=ps
   prs=ps
   pil=pi
   pbl=pb
   psl=ps
   prl=ps
   pr=ps
   pcf=pc
   pcs=pc
   pcl=pc
endif

save,in,bl,sp,c,ch,re,ca,pi,pb,ps,pc,pr,pif,pbf, $
     	psf,pcf,prf,pis,pbs,pss,pcs,prs,pil,pbl,psl,pcl,prl, $
        wlm_ref_time,wlm_ntel,wlm_itel,       $
        wlm_times,wlm_id,wlm_raw,wlm_wts,     $
        wlm_start,wlm_stop,wlm_cal,           $
        wlm_gains,wlm_egains,wlm_xfactor,     $
        wlm_filter,wlm_icoherence,            $
        pl,s_page,gai,pas,con,spe,wlm,map,var, $
    	filename=filename,/verbose

print,'If there were no error messages, the data was saved to: '
print,filename
return

end
